(******************************************************************************)
(* TopSpeed compatible file i/o                                               *)
(* (C) 2009 Steffen Solyga <solyga@absinth.net>                               *)
(******************************************************************************)
DEFINITION MODULE FIO;

FROM SYSTEM IMPORT BYTE;

TYPE
  File = CARDINAL;

CONST
  noFile = MAX( CARDINAL );

  StandardInput  = 0;
  StandardOutput = 1;
  ErrorOutput    = 2;

  resOk = 0;

(* controllers *)
VAR
  IOcheck : BOOLEAN;	(* if TRUE errors terminate program with report *)
  ChopOff : BOOLEAN;
  PrefixChar : CHAR;

CONST
  defIOcheck = TRUE;
  defChopOff = FALSE;
  defPrefixChar = ' ';

(* additional return values *)
VAR
  OK : BOOLEAN;	(* read & write: exactly as many bytes processed as requested *)
  EOF: BOOLEAN;	(* read only: attmpt to read beyond last byte (which isn't OK) *)

(* Differences to TopSpeed's implementation:
 * 1. In contrast to the TopSpeed implementation, all open array parameter
 *    procs check the array size; the number of bytes requested is the minimum
 *    of this size and the explicit count (if any).
 * 2. TopSpeed doesn't use VAR for open arrays which should stay constant
 *    (WrBin, WrStr), but they switch open array copying off. Mocka doesn't
 *    have this option, so I'm using VAR for speedup.
 *)

PROCEDURE IOresult(): CARDINAL;

PROCEDURE Open(     name: ARRAY OF CHAR ): File;
PROCEDURE OpenRead( name: ARRAY OF CHAR ): File;
PROCEDURE Create(   name: ARRAY OF CHAR ): File;
PROCEDURE Append(   name: ARRAY OF CHAR ): File;
PROCEDURE Close(        f: File );
PROCEDURE Size(         f: File ): LONGCARD;
(* not yet implemented since not in mocka's SysLib
PROCEDURE GetPos(       f: File ): LONGCARD;
PROCEDURE Seek(         f: File; p: LONGCARD );
*)
PROCEDURE AssignBuffer( f: File; VAR b: ARRAY OF BYTE );

PROCEDURE WrBin(        f: File; VAR b: ARRAY OF BYTE; n: CARDINAL );
PROCEDURE WrStr(        f: File; VAR s: ARRAY OF CHAR );
PROCEDURE WrChar(       f: File; c: CHAR );
PROCEDURE WrCharRep(    f: File; c: CHAR; n: CARDINAL );
PROCEDURE WrLn(         f: File );
PROCEDURE WrStrAdj(     f: File; VAR s: ARRAY OF CHAR; l: INTEGER );
PROCEDURE WrShtCard(    f: File; x: SHORTCARD; l: INTEGER );
PROCEDURE WrCard(       f: File; x: CARDINAL;  l: INTEGER );
PROCEDURE WrLngCard(    f: File; x: LONGCARD;  l: INTEGER );
PROCEDURE WrShtHex(     f: File; x: SHORTCARD; l: INTEGER );
PROCEDURE WrHex(        f: File; x: CARDINAL;  l: INTEGER );
PROCEDURE WrLngHex(     f: File; x: LONGCARD;  l: INTEGER );

PROCEDURE RdBin(        f: File; VAR b: ARRAY OF BYTE; n: CARDINAL) : CARDINAL;
PROCEDURE RdChar(       f: File ): CHAR;

END FIO.
